﻿using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Operators;
using HeuristicLab.Optimization;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Problems.MetaOptimization {
  /// <summary>
  /// An operator which creates a new random permutation of integer values.
  /// </summary>
  [Item("RandomParameterVectorCreator", "An operator which creates a new set of parameters TODO.")]
  [StorableClass]
  public sealed class RandomParameterConfigurationCreator : SingleSuccessorOperator, IStochasticOperator, IParameterConfigurationCreator {
    public override bool CanChangeName {
      get { return false; }
    }

    public ILookupParameter<IRandom> RandomParameter {
      get { return (LookupParameter<IRandom>)Parameters["Random"]; }
    }
    public ILookupParameter<IValueConfiguration> ValueConfigurationParameter {
      get { return (ILookupParameter<IValueConfiguration>)Parameters["ValueConfiguration"]; }
    }

    public ILookupParameter<IValueConfiguration> NewValueConfigurationParameter {
      get { return (ILookupParameter<IValueConfiguration>)Parameters["NewValueConfiguration"]; }
    }

    [StorableConstructor]
    private RandomParameterConfigurationCreator(bool deserializing) : base(deserializing) { }
    private RandomParameterConfigurationCreator(RandomParameterConfigurationCreator original, Cloner cloner) : base(original, cloner) { }
    public RandomParameterConfigurationCreator() : base() {
      Parameters.Add(new LookupParameter<IRandom>("Random", "The pseudo random number generator which should be used to initialize the new random permutation."));
      Parameters.Add(new LookupParameter<IValueConfiguration>("ValueConfiguration", "The new random parameter set."));
    }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new RandomParameterConfigurationCreator(this, cloner);
    }

    public override IOperation Apply() {
      NewValueConfigurationParameter.ActualValue = (IValueConfiguration)ValueConfigurationParameter.ActualValue.Clone();
      NewValueConfigurationParameter.ActualValue.Randomize();
      NewValueConfigurationParameter.ActualValue.Parameterize();
      return base.Apply();
    }
  }
}
